PROGRAM_NAME='Met-Ecom_Snapshot'
(***********************************************************)

#include 'Base_64_Encode'

/*

Directions:  
   1) Compile and download to the master
   2) Enable Diagnostics
   3) Push button 1 to take a snapshot

After you push button 1:
   1) The diagnostics window will show you the progress of the data retrieval.
   2) When transmission is complete a live snapshot from the Met-Ecom will be stored on the master in the form of a jpg.
   3) Also saved to flash will be the raw contents retrieved from the HTTP GET.
      
*/

DEFINE_DEVICE

dvIP	= 0:3:0
dvTP	= 10001:1:0


DEFINE_CONSTANT

INTEGER nWayTooMuch = 28000	// IP buffer bail-out threshhold


DEFINE_VARIABLE _

VOLATILE CHAR 		cBuffer[32000]		// we'll manipulate this one
VOLATILE CHAR 		cBufferCopy[32000]	// copy remains untouched and contents gets saved to flash
VOLATILE CHAR		cTopHeader[2048]
VOLATILE CHAR		cFrameHeader[2048]
VOLATILE CHAR		cImage[12000]		// this will get saved to flash as a jpg
VOLATILE INTEGER 	nOnline 
VOLATILE INTEGER 	nImageLength
VOLATILE INTEGER 	nLengthGood

#warn 'Set these next 3 to match the Met-Ecom IP address, user name, and password'
VOLATILE CHAR		cIPAddress[]	= '192.168.1.87'
VOLATILE CHAR		cUserName[]	= 'Admin'
VOLATILE CHAR		cPassword[]	= '1988'


DEFINE_FUNCTION fnSave() {

   SLONG 	handle
   CHAR	 	filename[32]
     
   filename = "'\METECOM_HTTP.TXT'"
   handle = FILE_OPEN(filename,FILE_RW_NEW)

   IF (handle>0) {
   
      FILE_WRITE(handle,cBufferCopy,LENGTH_ARRAY(cBufferCopy))
            
      FILE_CLOSE(handle)
   }
   
   filename = "'\METECOM_SNAPSHOT.JPG'"
   handle = FILE_OPEN(filename,FILE_RW_NEW)

   IF (handle>0) {
   
      cImage = GET_BUFFER_STRING(cBuffer,nImageLength)
     
      FILE_WRITE(handle,cImage,LENGTH_ARRAY(cImage))
            
      FILE_CLOSE(handle)
   }
   
}

DEFINE_START

CREATE_BUFFER dvIP,cBuffer
CREATE_BUFFER dvIP,cBufferCopy

DEFINE_EVENT

BUTTON_EVENT[dvTP,1] {	// take shapeshot

   PUSH: {
      
      IF (!nOnline) {
	 
	 cBuffer = ""
	 cBufferCopy = ""
	 cTopHeader = ""
	 cFrameHeader = ""
	 cImage = ""
	 nImageLength = 0
	 nLengthGood = 0
	 
	 SEND_STRING 0, "'Connecting to Met-Ecom at IP Address: ',cIPAddress"
	 IP_CLIENT_OPEN(dvIP.PORT,cIPAddress,80,IP_TCP)
      }
   }

}

DATA_EVENT[dvIP] {


   ONLINE: {

      CHAR outstring[256]
      
      nOnline = 1

      outstring = "'GET /pages/stream_cam_wg.mjpeg?source=1 HTTP/1.1',13,10"
      outstring = "outstring,'Authorization: Basic ',EncrBase64Encode("cUserName,':',cPassword"),13,10"
      outstring = "outstring,13,10"  // indicate end of header
   
      SEND_STRING 0,"'Met-Ecom Online - Sending Stream Request'"
      SEND_STRING dvIP,"outstring"
      
   }
   
   STRING: {
            
      IF (FIND_STRING(cBuffer,"13,10,13,10",1) && cTopHeader="") {

	 SEND_STRING 0,"'Getting Top Header...'"
	 cTopHeader = REMOVE_STRING(cBuffer,"13,10,13,10",1)
	 	 
      }
      
      IF (FIND_STRING(cBuffer,"13,10,13,10",1) && LENGTH_ARRAY(cTopHeader) && nImageLength=0) {
      
	 SEND_STRING 0,"'Getting Frame Header...'"
	 cFrameHeader = REMOVE_STRING(cBuffer,"13,10,13,10",1)

	 IF (FIND_STRING(cFrameHeader,"'Content-Length:'",1)) {
	    
	    SEND_STRING 0,"'Getting Length...'"
	    REMOVE_STRING(cFrameHeader,"'Content-Length:'",1)
	    
	    nImageLength = ATOI(cFrameHeader)
	    SEND_STRING 0,"'Length of Frame is: ',ITOA(nImageLength)"
	 }
	 
      }
      
      IF (LENGTH_ARRAY(cBuffer) >=nImageLength && nImageLength>0) {
	 
	 SEND_STRING 0,"'Got Image - Closing IP Port...'"
	 nLengthGood = 1	 
	 IP_CLIENT_CLOSE(dvIP.PORT)
      }
      
      IF (LENGTH_ARRAY(cBuffer)>nWayTooMuch) {
	 
	 SEND_STRING 0, 'Something got mucked up so let''s get out of here.'
	 IP_CLIENT_CLOSE(dvIP.PORT)
      }
   }
   
   OFFLINE: {
      
      nOnline = 0
      SEND_STRING 0,"'MetEcom Offline - Saving to Flash'"
      SEND_STRING 0, "'nLengthGood=',ITOA(nLengthGood)"
      fnSave()
   
   }

}


